{% extends "base.html" %}

{% block title %}Welcome to Clay{% endblock %}

{% block head %}
{# The styles linked here are only for the demo page #}
<link href="/static/styles/index.css" rel="stylesheet">
{%- endblock %}

{% block content %}
  <h1>Welcome to Clay</h1>
  <p>Now open <code>source/index.html</code> to edit me. ;)</p>
{%- endblock %}
