# -*- coding: utf-8 -*-
"""
# Clay

An amazing prototyping tool

------------
© 2011 by [Lúcuma](http://lucumalabs.com).

See `AUTHORS.md` for more details.

[MIT License](http://www.opensource.org/licenses/mit-license.php)

"""
from main import Clay


__version__ = '2.7.1'
